@@ -27,6 +27,7 @@
 #include "mongoc-rand-private.h"
 #include "mongoc-util-private.h"
 #include "mongoc-client.h"
+#include "mongoc-client-private.h" // WIRE_VERSION_* macros.
 #include "mongoc-client-session-private.h"
 #include "mongoc-trace-private.h"
 
@@ -70,10 +71,10 @@ _mongoc_hex_md5 (const char *input)
    char digest_str[33];
    int i;
 
-   COMMON_PREFIX (_bson_md5_init (&md5));
-   COMMON_PREFIX (_bson_md5_append (
-      &md5, (const uint8_t *) input, (uint32_t) strlen (input)));
-   COMMON_PREFIX (_bson_md5_finish (&md5, digest));
+   mcommon_md5_init (&md5);
+   mcommon_md5_append (
+      &md5, (const uint8_t *) input, (uint32_t) strlen (input));
+   mcommon_md5_finish (&md5, digest);
 
    for (i = 0; i < sizeof digest; i++) {
       bson_snprintf (&digest_str[i * 2], 3, "%02x", digest[i]);
@@ -288,24 +289,30 @@ _mongoc_wire_version_to_server_version (int32_t version)
       return "3.4";
    case 6:
       return "3.6";
-   case 7:
+   case WIRE_VERSION_4_0:
       return "4.0";
-   case 8:
+   case WIRE_VERSION_4_2:
       return "4.2";
-   case 9:
+   case WIRE_VERSION_4_4:
       return "4.4";
    case 10:
       return "4.7";
    case 11:
       return "4.8";
-   case 12:
+   case WIRE_VERSION_4_9:
       return "4.9";
-   case 13:
+   case WIRE_VERSION_5_0:
       return "5.0";
-   case 14:
+   case WIRE_VERSION_5_1:
       return "5.1";
    case 15:
       return "5.2";
+   case 16:
+      return "5.3";
+   case WIRE_VERSION_6_0:
+      return "6.0";
+   case WIRE_VERSION_7_0:
+      return "7.0";
    default:
       return "Unknown";
    }
@@ -572,10 +579,10 @@ _mongoc_bson_array_copy_labels_to (const bson_t *reply, bson_t *dst)
 
 /*--------------------------------------------------------------------------
  *
- * _mongoc_bson_init_with_transient_txn_error --
+ * _mongoc_add_transient_txn_error --
  *
- *       If @reply is not NULL, initialize it. If @cs is not NULL and in a
- *       transaction, add errorLabels: ["TransientTransactionError"] to @cs.
+ *       If @cs is not NULL and in a transaction, add errorLabels:
+ *       ["TransientTransactionError"] to @reply.
  *
  *       Transactions Spec: TransientTransactionError includes "server
  *       selection error encountered running any command besides
@@ -590,21 +597,27 @@ _mongoc_bson_array_copy_labels_to (const bson_t *reply, bson_t *dst)
  */
 
 void
-_mongoc_bson_init_with_transient_txn_error (const mongoc_client_session_t *cs,
-                                            bson_t *reply)
+_mongoc_add_transient_txn_error (const mongoc_client_session_t *cs,
+                                 bson_t *reply)
 {
-   bson_t labels;
-
    if (!reply) {
       return;
    }
 
-   bson_init (reply);
-
    if (_mongoc_client_session_in_txn (cs)) {
-      BSON_APPEND_ARRAY_BEGIN (reply, "errorLabels", &labels);
-      BSON_APPEND_UTF8 (&labels, "0", TRANSIENT_TXN_ERR);
-      bson_append_array_end (reply, &labels);
+      bson_t labels = BSON_INITIALIZER;
+      _mongoc_bson_array_copy_labels_to (reply, &labels);
+      _mongoc_bson_array_add_label (&labels, TRANSIENT_TXN_ERR);
+
+      bson_t new_reply = BSON_INITIALIZER;
+      bson_copy_to_excluding_noinit (reply, &new_reply, "errorLabels", NULL);
+      BSON_APPEND_ARRAY (&new_reply, "errorLabels", &labels);
+
+      bson_reinit (reply);
+      bson_concat (reply, &new_reply);
+
+      bson_destroy (&labels);
+      bson_destroy (&new_reply);
    }
 }
 
@@ -665,9 +678,9 @@ _mongoc_getenv (const char *name)
       return NULL;
    }
 #else
-
-   if (getenv (name) && strlen (getenv (name))) {
-      return bson_strdup (getenv (name));
+   char *const var = getenv (name);
+   if (var && strlen (var)) {
+      return bson_strdup (var);
    } else {
       return NULL;
    }
@@ -863,3 +876,74 @@ _mongoc_rand_size_t (size_t min, size_t max, size_t (*rand) (void))
    "Implementation of _mongoc_simple_rand_size_t() requires size_t be exactly 32-bit or 64-bit"
 
 #endif
+
+bool
+_mongoc_iter_document_as_bson (const bson_iter_t *iter,
+                               bson_t *bson,
+                               bson_error_t *error)
+{
+   uint32_t len;
+   const uint8_t *data;
+
+   if (!BSON_ITER_HOLDS_DOCUMENT (iter)) {
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
+                      MONGOC_ERROR_COMMAND_INVALID_ARG,
+                      "expected BSON document for field: %s",
+                      bson_iter_key (iter));
+      return false;
+   }
+
+   bson_iter_document (iter, &len, &data);
+   if (!bson_init_static (bson, data, len)) {
+      bson_set_error (error,
+                      MONGOC_ERROR_COMMAND,
+                      MONGOC_ERROR_COMMAND_INVALID_ARG,
+                      "unable to initialize BSON document from field: %s",
+                      bson_iter_key (iter));
+      return false;
+   }
+
+   return true;
+}
+
+uint8_t *
+hex_to_bin (const char *hex, uint32_t *len)
+{
+   uint8_t *out;
+
+   const size_t hex_len = strlen (hex);
+   if (hex_len % 2u != 0u) {
+      return NULL;
+   }
+
+   BSON_ASSERT (bson_in_range_unsigned (uint32_t, hex_len / 2u));
+
+   *len = (uint32_t) (hex_len / 2u);
+   out = bson_malloc0 (*len);
+
+   for (uint32_t i = 0; i < hex_len; i += 2u) {
+      uint32_t hex_char;
+
+      if (1 != sscanf (hex + i, "%2x", &hex_char)) {
+         bson_free (out);
+         return NULL;
+      }
+
+      BSON_ASSERT (bson_in_range_unsigned (uint8_t, hex_char));
+      out[i / 2u] = (uint8_t) hex_char;
+   }
+   return out;
+}
+
+char *
+bin_to_hex (const uint8_t *bin, uint32_t len)
+{
+   char *out = bson_malloc0 (2u * len + 1u);
+
+   for (uint32_t i = 0u; i < len; i++) {
+      bson_snprintf (out + (2u * i), 3, "%02x", bin[i]);
+   }
+
+   return out;
+}
