@@ -257,7 +257,7 @@ test_mongoc_handshake_data_append_success (void)
       ASSERT (strstr (val, platform) != NULL);
    }
 
-   mock_server_replies_simple (request, "{'ok': 1, 'isWritablePrimary': true}");
+   reply_to_request_simple (request, "{'ok': 1, 'isWritablePrimary': true}");
    request_destroy (request);
 
    /* Cleanup */
@@ -367,7 +367,7 @@ test_mongoc_handshake_data_append_null_args (void)
       ASSERT (strstr (val, "null") == NULL);
    }
 
-   mock_server_replies_simple (request, "{'ok': 1, 'isWritablePrimary': true}");
+   reply_to_request_simple (request, "{'ok': 1, 'isWritablePrimary': true}");
    request_destroy (request);
 
    /* Cleanup */
@@ -525,7 +525,7 @@ test_mongoc_handshake_too_big (void)
    /* Should have truncated the platform field so it fits exactly */
    ASSERT (len == HANDSHAKE_MAX_SIZE);
 
-   mock_server_replies_simple (
+   reply_to_request_simple (
       request,
       tmp_str ("{'ok': 1, 'minWireVersion': %d, 'maxWireVersion': %d}",
                WIRE_VERSION_MIN,
@@ -535,7 +535,7 @@ test_mongoc_handshake_too_big (void)
    request = mock_server_receives_msg (
       server, MONGOC_MSG_NONE, tmp_bson ("{'$db': 'admin', 'ping': 1}"));
 
-   mock_server_replies_simple (request, "{'ok': 1}");
+   reply_to_request_simple (request, "{'ok': 1}");
    ASSERT (future_get_bool (future));
 
    future_destroy (future);
@@ -562,11 +562,10 @@ test_mongoc_platform_truncate (int drop)
    char *undropped;
    char *expected;
    char big_string[HANDSHAKE_MAX_SIZE];
-   int handshake_remaining_space;
 
    /* Need to know how much space storing fields in our BSON will take
     * so that we can make our platform string the correct length here */
-   int handshake_bson_size = 163;
+   const size_t handshake_bson_size = 163;
    _reset_handshake ();
 
    md = _mongoc_handshake_get ();
@@ -590,8 +589,8 @@ test_mongoc_platform_truncate (int drop)
    bson_free (md->flags);
    md->flags = bson_strdup ("test_h");
 
-   handshake_remaining_space =
-      HANDSHAKE_MAX_SIZE -
+   size_t handshake_remaining_space =
+      (size_t) HANDSHAKE_MAX_SIZE -
       (strlen (md->os_type) + strlen (md->os_name) + strlen (md->os_version) +
        strlen (md->os_architecture) + strlen (md->driver_name) +
        strlen (md->driver_version) + strlen (md->compiler_info) +
@@ -610,8 +609,8 @@ test_mongoc_platform_truncate (int drop)
       undropped = bson_strdup_printf ("%s%s", md->compiler_info, md->flags);
    }
 
-   memset (big_string, 'a', handshake_remaining_space + 1);
-   big_string[handshake_remaining_space + 1] = '\0';
+   memset (big_string, 'a', handshake_remaining_space + 1u);
+   big_string[handshake_remaining_space + 1u] = '\0';
 
    ASSERT (mongoc_handshake_data_append (NULL, NULL, big_string));
    ASSERT (_mongoc_handshake_build_doc_with_application (&doc, "my app"));
@@ -686,13 +685,13 @@ test_mongoc_handshake_cannot_send (void)
    ASSERT (request_doc);
    ASSERT (!bson_has_field (request_doc, HANDSHAKE_FIELD));
 
-   mock_server_replies_simple (request, server_reply);
+   reply_to_request_simple (request, server_reply);
    request_destroy (request);
 
    /* Cause failure on client side */
    request = mock_server_receives_any_hello (server);
    ASSERT (request);
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
 
    /* Make sure the hello request still DOESN'T have a handshake field
@@ -703,7 +702,7 @@ test_mongoc_handshake_cannot_send (void)
    ASSERT (request_doc);
    ASSERT (!bson_has_field (request_doc, HANDSHAKE_FIELD));
 
-   mock_server_replies_simple (request, server_reply);
+   reply_to_request_simple (request, server_reply);
    request_destroy (request);
 
    /* cleanup */
