@@ -443,7 +443,7 @@ hangup_except_hello (request_t *request, void *data)
       return false;
    }
 
-   mock_server_hangs_up (request);
+   reply_to_request_with_hang_up (request);
    request_destroy (request);
    return true;
 }
@@ -663,7 +663,7 @@ test_unknown_commit_result (void)
       collection, tmp_bson ("{}"), &opts, NULL, &error);
    request = mock_server_receives_msg (
       server, 0, tmp_bson ("{'insert': 'collection'}"), tmp_bson ("{}"));
-   mock_server_replies_ok_and_destroys (request);
+   reply_to_request_with_ok_and_destroy (request);
    ASSERT_OR_PRINT (future_get_bool (future), error);
    future_destroy (future);
 
@@ -947,7 +947,6 @@ test_selected_server_is_pinned_to_mongos (void *ctx)
    uint32_t expected_id;
    uint32_t actual_id;
    const mongoc_server_description_t *sd = NULL;
-   int i;
 
    BSON_UNUSED (ctx);
 
@@ -1006,7 +1005,7 @@ test_selected_server_is_pinned_to_mongos (void *ctx)
    /* get a valid server id that's different from the pinned server id */
    servers =
       mc_tpld_servers_const (mc_tpld_unsafe_get_const (client->topology));
-   for (i = 0; i < servers->items_len; i++) {
+   for (size_t i = 0; i < servers->items_len; i++) {
       sd = mongoc_set_get_item_const (servers, i);
       if (sd && sd->id != actual_id) {
          break;
